// ---------------------------------------------------------------------
// MyLinkedList.cpp
//
// Written by Brent Dingle
// Last Modified: 2013
//
// Example of a linked list and its associated Node type
// Technically class Node should be declared separately
// using its own header and .cpp file
// We will use it as struct for the most part
// ---------------------------------------------------------------------
#include <iostream>
#include <string>
#include "MyLinkedList.h"

using namespace std;

// ---------------------------------------------------------------------
// Constructor
// ---------------------------------------------------------------------
MyLinkedList::MyLinkedList()
{
	// Begin with an empty list
	mp_startPtr = NULL;
}

// ---------------------------------------------------------------------
// Destructor
// ---------------------------------------------------------------------
MyLinkedList::~MyLinkedList()
{
	// Remove all nodes
	while (mp_startPtr != NULL)
	{
		removeFront();
	}
}

// ---------------------------------------------------------------------
// isEmpty
// returns true if list is empty
// ---------------------------------------------------------------------
bool MyLinkedList::isEmpty() const
{
	bool retVal = false;
	if (mp_startPtr == NULL)
	{
		retVal = true;
	}
	return retVal;
}

// ---------------------------------------------------------------------
// findNode
// returns null or node w/ matching name
// ---------------------------------------------------------------------
Node* MyLinkedList::findNode(string findName)
{
	Node* currentPtr = mp_startPtr;
	
    while (   (currentPtr != NULL )
           &&  (currentPtr->m_name != findName) )
    {
        currentPtr = currentPtr->mp_next;
    }
	return currentPtr;
}

// ---------------------------------------------------------------------
// addNode
// add node to list - defaults to adding to the beginning of the list
// If we were trying to maintain a sorted list we would likely
// find the correct place to insert newNode to maintain the
// order of the list (rather than sort it agani and again)
// ---------------------------------------------------------------------
void MyLinkedList::addNode(const Node& newNode)
{
	// Make a copy of the read-only node sent
	// Should NOT just point to it
	// If class Node has a copy constructor or assignment operator
	// overloaded this becomes much easier.
	// Assume it does not, and do this the 'hard' way
	Node* nodeCopy = new Node;
	nodeCopy->m_name = newNode.m_name;
	nodeCopy->m_age  = newNode.m_age;
	
	// make nodeCopy's next be the current head of the list
	nodeCopy->mp_next = mp_startPtr;
	
	// And make nodeCopy be the new head of the list
	mp_startPtr = nodeCopy;
}

// ---------------------------------------------------------------------
// removeFront
// removes first node of list
// and deletes its memory
// ---------------------------------------------------------------------
void MyLinkedList::removeFront()
{
    Node* oldHeadPtr = mp_startPtr;
    mp_startPtr = oldHeadPtr->mp_next;
    delete oldHeadPtr;
}

// ---------------------------------------------------------------------
// remove
// Remove any and all nodes with a name matching the one sent
// ---------------------------------------------------------------------
void MyLinkedList::remove(string name)
{
    // We need to know the node that comes BEFORE the node the
    // name is found at as well as the node with the
    // matching name... a time when a doubly linked list
    // becomes useful... but we don't have
    
    Node* prevPtr = NULL;
	Node* curPtr  = mp_startPtr;
	
	// Note we must go through the entire list to remove ALL
	// occurrances of name
    while (curPtr != NULL )
    {
		if (curPtr->m_name == name)
		{
			// remove the node
			if (prevPtr != NULL)  
			{
			    prevPtr->mp_next = curPtr->mp_next;
                delete curPtr;
                curPtr = prevPtr->mp_next;
			}
			else // prevPtr == NULL so name match was on head of list
			{
				prevPtr = curPtr->mp_next;  // this will be the new head
                delete curPtr;
                curPtr = prevPtr;   // the new head
                mp_startPtr = curPtr;
                prevPtr = NULL;     // and nothing comes before the head
			}
		}
		else  // no name match on current node
		{
		    prevPtr = curPtr;
            curPtr  = curPtr->mp_next;
		}
    } // end while
}

// ---------------------------------------------------------------------
// printList
// Display the list to the screen from head to tail
// ---------------------------------------------------------------------
void MyLinkedList::printList()
{
	Node* curPtr = mp_startPtr;
	
	if (curPtr == NULL)
	{
		cout << "List is empty.\n";
	}
	else
	{
		cout << "The List:\n";
	}
	
	while (curPtr != NULL)
	{
		cout << "   " << curPtr->m_name << ", age " << curPtr->m_age << endl;
		curPtr = curPtr->mp_next;
	}
}

// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// end file
