// ---------------------------------------------------------------------
// TestMyLinkedList.cpp
//
// Written by Brent Dingle
// Last Modified: 2013
//
// Example of a linked list and its associated Node type
// Technically class Node should be declared separately
// using its own header and .cpp file
// We will use it as struct for the most part
// ---------------------------------------------------------------------
#include <iostream>
#include <string>
#include "MyLinkedList.h"

using namespace std;

// ---------------------------------------------------------------------
// main function --- tis is where it all begins
// ---------------------------------------------------------------------
int main()
{
	MyLinkedList myList;
	
	if (myList.isEmpty())
	{
		cout << "isEmpty() --- passed\n";
	}
	else
	{
		cout << "isEmpty() --- failed\n";
	}
	
    // ----------------------------------------------	
	cout << "Creating first node.\n";
	Node tempNode;
	tempNode.m_name = "Robert";
	tempNode.m_age  = 105;
	cout << "Adding first node.\n";
	myList.addNode(tempNode);
	cout << "Printing out the list.\n";
	myList.printList();

    // ----------------------------------------------	
	cout << "Adding second node.\n";
	tempNode.m_name = "Julia";
	tempNode.m_age  = 68;
	myList.addNode(tempNode);
	cout << "Printing out the list.\n";
	myList.printList();

    // ----------------------------------------------	
	cout << "Adding third node.\n";
	tempNode.m_name = "Jacks";
	tempNode.m_age  = 23;
	myList.addNode(tempNode);

    // ----------------------------------------------	
	cout << "Adding fourth node.\n";
	tempNode.m_name = "Doctor";
	tempNode.m_age  = 4503;
	myList.addNode(tempNode);

    // ----------------------------------------------	
	cout << "Adding fifth node.\n";
	tempNode.m_name = "Doctor";
	tempNode.m_age  = 3659;
	myList.addNode(tempNode);

    // ----------------------------------------------	
	cout << "Adding sixth node.\n";
	tempNode.m_name = "Julia";
	tempNode.m_age  = 34;
	myList.addNode(tempNode);
	cout << "Printing out the list.\n";
	myList.printList();


    // ----------------------------------------------
    Node *findJ = myList.findNode("Julia");
    if (findJ == NULL)
    {
		cout << "findNode()  ---- failed.\n";
	}
	else
	{
		cout << "findNode() ---- found something.\n";
		cout << "name should be Julia, age should be 34\n";
		cout << "name = " << findJ->m_name << " age = " << findJ->m_age << endl;
	}

    // ----------------------------------------------
    cout << "Removing Doctor\n";
    myList.remove("Doctor");	
    // ----------------------------------------------
    cout << "Finding Doctor... should not find him\n";
    Node *findD = myList.findNode("Doctor");
    if (findD == NULL)
    {
		cout << "findNode --- passed\n";
	}
	else
	{
		cout << "findNode() ---- FAILED --- found something.\n";
		cout << "name = " << findD->m_name << " age = " << findD->m_age << endl;
	}

    // ----------------------------------------------
    cout << "Removing Julia\n";
    myList.remove("Julia");	
    // ----------------------------------------------
    cout << "Finding Julia... should not find her\n";
    Node *findJ2 = myList.findNode("Julia");
    if (findJ2 == NULL)
    {
		cout << "findNode --- passed\n";
		// if you crash/fail after this... (or crash when program ends)
		// may not have kept the mp_StartPtr accurate
	}
	else
	{
		cout << "findNode() ---- FAILED --- found something.\n";
		cout << "name = " << findJ2->m_name << " age = " << findJ2->m_age << endl;
	}

    // ----------------------------------------------
    cout << "Removing Robert\n";
    myList.remove("Robert");	
    // ----------------------------------------------
    cout << "Finding Robert... should not find him\n";
    Node *findR = myList.findNode("Robert");
    if (findR == NULL)
    {
		cout << "findNode --- passed\n";
	}
	else
	{
		cout << "findNode() ---- FAILED --- found something.\n";
		cout << "name = " << findR->m_name << " age = " << findR->m_age << endl;
	}

    // ----------------------------------------------
    cout << "Checking is empty --- should still have Jacks" << endl;
	if (myList.isEmpty())
	{
		cout << "isEmpty() says true --- failed\n";
	}
	else
	{
		cout << "isEmpty() says false --- passed\n";
	}

    // ----------------------------------------------
	cout << "Printing out the list.\n";
	myList.printList();

    // ----------------------------------------------
    cout << "Calling removeFront\n";
    myList.removeFront();
    
    cout << "Finding Jacks... should not find him\n";
    Node *findJx = myList.findNode("Jacks");
    if (findR == NULL)
    {
		cout << "findNode --- passed\n";
	}
	else
	{
		cout << "findNode() ---- FAILED --- found something.\n";
		cout << "name = " << findJx->m_name << " age = " << findJx->m_age << endl;
	}

    // ----------------------------------------------
	cout << "Printing out the list -- expect it to be empty.\n";
	myList.printList();

    // Add more testing after this or elsewhere as needed
    
}

// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// end file
